#' Penalized multiple Co-Inertia Analysis for 
#' 
#' This function conduct penalized multiple co-inertia analysis for a given set of tuning parameter values. This function esimate one set of loading vectors for a given datasets.
#' To estimate consequent loading vectors, put deflated datasets using previously estimated sets of loading vectors. 
#' @param data A length-K list object containing data matrices \mathbf{X}_1, \ldots, \mathbf{X}_K.
#' @param gridpt A J-by-(K+2) matrix containing combinations of tuning parameters to investigate. Each row is one combination of (\lambda,stepsize, s_1,\ldots, s_K).
#' @param nfold Number of subgroups for the cross validation.
#' @param tL A length K list object containing transformed laplacian matrix for network penalties. k-th element of \code{tL} contains a transformed laplacian matrix of the network information for the k-th data matrix.
#' @param n.penalty Indicator variable to let the function know whether user wants to incorporate structured information existing among data matrices. Default value is \code{FALSE}.
#' @param prop.test A scalar between 0 and 1, it is  proportion of test data among the whole data, if user want to set aside the test dataset. Default value is 0. 
#' @param npc A scaler value, a number of sets of loading vectors to estimate. Default value is 1. 
#' @param savefile 
#' @return This function returns a list object "opt.results" containing npc number of lists named "opt.result". Each list contains following elements:
#' \item{a}{A length-K list containing the estimated loading vectors of K data matrices.}
#' \item{b}{A n-by-1 vector, the estimated synthetic reference vector.}
#' \item{test.objval/whole.objval}{Calculated objective value using the estimated loading vectors. If prop.test is nonzero, objective value is calculated using the test data.}
#' \item{opt.tune}{A combination of optimal tuning parameter selected by cross validation.}
#' @seealso \code{\link{mCIA}}, \code{\link{l0pmCIA}}, \code{\link{genGridpt}}, \code{\link{plot.pmCIA}}
#' @examples
#' library(pmCIA)
#' data(demoData)
#' s.gridpt <- genGridpt(s=c(50, 50, 50), 
#'                       edit = list(matrix(c(-10, 0, 10), ncol=1),matrix(c(-10, 0, 10), ncol=1),
#'                       matrix(c(-10, 0, 10), ncol=1)), 
#'                       n.steps=c(0,1,5))
#' n.gridpt <- genGridpt(s=c(50, 50, 50),
#'                        edit = list(matrix(c(-10, 0, 10), ncol=1),matrix(c(-10, 0, 10), ncol=1),
#'                        matrix(c(-10, 0, 10), ncol=1)),
#'                        n.steps=c(0,1,5), 
#'                        n.penalty = T, 
#'                        info.lambda = c(0,1,5,1))
#' single.data <- list(demoData$X1[1:200,],demoData$X2[1:200,], demoData$X3[1:200,])
#' res.sparse <- cv.l0pmCIA(singledata, s.gridpt)
#' res.network <- cv.l0pmCIA(singedata, n.gridpt, tL=demoData$tL, n.penalty=T)
#' @export
cv.l0pmCIA <- function(data, gridpt, nfold=5, tL=NULL, n.penalty=FALSE, prop.test=0, npc=1, savefile=NULL)
{
  set.seed(12345)
  n <- dim(data[[1]])[1] # sample size
  ps <- unlist(lapply(data, function(x) dim(x)[2]) )   # number of datasets
  # if user wants to preserve the test dataset separately
  if (prop.test!=0)
  {
    testgrp.id <- sample(nrow(data[[1]]), round(n*prop.test))
    subgrp.id  <- sample(rep(1:nfold, length.out=n-round(n*prop.test)))
    test.data  <- lapply(data, function(x) x[testgrp.id,])
    cv.data    <- lapply(data, function(x) x[-testgrp.id,])
  }else{
    subgrp.id <- sample(rep(1:nfold, length.out=n))
    cv.data   <- data
  }
  # standardize and transform the data
  cv.sdata <- lapply(cv.data, stdDat)
  ds <- lapply(cv.sdata, function(x) apply(abs(x), 2, sum))
  sQ <- lapply(cv.sdata, function(x) diag(sqrt(apply(abs(x), 2, sum))))
  cv.tdata <- list()
  for (i in 1:ps) cv.tdata[[i]] <- cv.sdata[[i]]%*%sQ[[i]]
  orig.cv.Tdat <- cv.Tdat <- matrix(unlist(cv.tdata), nrow=n)
  if (prop.test!=0)
  {
    test.sdata <- lapply(test.data, stdDat)
    for (i in 1:ps) test.tdata <- test.sdata[[i]] %*% sQ[[i]]
    orig.test.Tdat <- test.Tdat <- matrix(unlist(test.tdata), nrow=n)
  }
  # generate transformed laplacian matrix if an edgelist is given
  if (!is.null(tL))
  {
    for (i in 1:ps)
    {
      E   <- tL[[i]]
      tmp <- igraph::graph_from_edgelist(E, directed = FALSE)
      gx  <- igraph::add_vertices(tmp, ps[i]-length(V(tmp)))
      L   <- igraph::laplacian_matrix(gx, normalized=T, weights=NULL, sparse = F)
      tL[[i]] <- diag(1/sqrt(ds1)) %*% L %*% diag(1/sqrt(ds1))
    }
  }
  # start algorithm
  opt.results <- list()
  for (pc in 1:npc) # estimate npc sets of loading vectors
  {
    cvobj <- rep(0, dim(gridpt)[1])
    for (j in 1:dim(gridpt)[1])
    {
      objvals <- NULL
      for (k in 1:nfold) # cross validation
      {
        # fit training data
        Tdat.train <- cv.Tdat[!(subgrp.id %in% k),]
        Tdat.val <- cv.Tdat[subgrp.id %in% k,]
        res.train <- l0pmCIA(Tdat.train, ps, s=gridpt[j,3:5],
                             step=gridpt[j,2],
                             n.penalty=n.penalty,
                             tL=tL,
                             lambda=gridpt[j,1],
                             threshold = 1e-4, max_iter=300)
        objvals <- c(objvals, cal.obj(res.train, Tdat.val, ps))
      }
      print(paste("gridpt=", j," is done", sep=""))
      cvobj[j] <- mean(objvals)/sd(objvals)
    }
    # select the best tuning parameter combination
    best.ind <- which.max(cvobj)
    # estimate loading vectors using chosen tuning parameters
    if (prop.test!=0)
    {
      res <- l0pmCIA(cv.Tdat, ps, s=gridpt[best.ind,3:5],
                     step=gridpt[best.ind,2],
                     n.penalty=n.penalty,
                     tL=tL,
                     lambda=gridpt[best.ind,1],
                     threshold = 1e-4, max_iter=300)
      opt.result <- list("a"=res$a, "b"=res$b,
                         "test.objval"=cal.obj(res, orig.test.Tdat, ps),
                         "opt.tune"=gridpt[best.ind,])
    }else{
      res <- l0pmCIA(cv.Tdat, ps, s=gridpt[best.ind,3:5],
                     step=gridpt[best.ind,2],
                     n.penalty=n.penalty,
                     tL=tL,
                     lambda=gridpt[best.ind,1],
                     threshold = 1e-4, max_iter=300)
      opt.result <- list("a"=res$a, "b"=res$b,
                         "whole.objval"=cal.obj(res, orig.cv.Tdat, ps),
                         "opt.tune"=gridpt[best.ind,])
    }
    opt.results[[pc]] = opt.result
    # deflation if npc > 2
    if (pc > 1)
    {
      for (def in 1:pc-1)
      {
        projmat <- lapply(opt.results[[def]]$a, get.projmat)
        for (i in 1:ps) cv.tdata[[i]] <- cv.tdata[[i]] %*% (diag(ps[i] - projmat[[i]]))
        if (length(prop.test)!=0) for (i in 1:ps) test.tdata <- test.tdata[[i]] %*% (diag(ps[i] - projmat[[i]]))
      }
      cv.Tdat <- matrix(unlist(cv.tdata), nrow=n)
      if (length(prop.test)!=0) test.Tdat <- matrix(unlist(test.tdata), nrow=n)
    }
  }
  # save result
  if(!is.null(savefile)) save(opt.results, file=savefile)
  return(opt.results)
}
