#' Generating Grid Points for Cross Valdiation to Select Optimal Tuning Parameters of Penalized Multple Co-Inertia Analysis
#' 
#' This function generates a set of grid points of possible tuning parameters for cross validation procedure of penalized mCIA. User can manually generate the set of grid points instead of using this function.
#' @param s A length-K vector, containing desired number of nonzero elements in the estimanted loading vectors of each K datasets.
#' @param edit A J-by-(K+2) matrix containing combinations of tuning parameters to investigate. Each row is one combination of (\lambda,stepsize, s_1,\ldots, s_K).
#' @param nsteps A vector with three elements, (v1, v2, v3). Using this input, this function generates v3 evenly-distributed number of stepsize between v1 and v2.
#' @param tL A length K list object containing transformed laplacian matrix for network penalties. k-th element of \code{tL} contains a transformed laplacian matrix of the network information for the k-th data matrix.
#' @param n.penalty Indicator variable to let the function know whether user wants to incorporate structured information existing among data matrices. Default value is \code{FALSE}.
#' @param info.lambda A vector with four elements, (v1, v2, v3, v4). Using this input, this function generates v3 number of lambda (tuning paramter for the network penalty) values between v1 and v2, with increaing rate v4.  
#' @return This function returns a matrix, each row is one combination of tuning parameters while each column stands for lambda, stepsize, desired number of nonzero elements in the estimated loading for the first data, ..., and desired number of nonzero elements in the estimated loading for K-th data.
#' @seealso \code{\link{mCIA}}, \code{\link{l0pmCIA}}, \code{\link{genGridpt}}, \code{\link{plot.pmCIA}}
#' @examples
#' library(pmCIA)
#' data(demoData)
#' s.gridpt <- genGridpt(s=c(50, 50, 50), 
#'                       edit = list(matrix(c(-10, 0, 10), ncol=1),matrix(c(-10, 0, 10), ncol=1),
#'                       matrix(c(-10, 0, 10), ncol=1)), 
#'                       n.steps=c(0,1,5))
#' n.gridpt <- genGridpt(s=c(50, 50, 50),
#'                        edit = list(matrix(c(-10, 0, 10), ncol=1),matrix(c(-10, 0, 10), ncol=1),
#'                        matrix(c(-10, 0, 10), ncol=1)),
#'                        n.steps=c(0,1,5), 
#'                        n.penalty = T, 
#'                        info.lambda = c(0,1,5,1))
#' single.data <- list(demoData$X1[1:200,],demoData$X2[1:200,], demoData$X3[1:200,])
#' res.sparse <- cv.l0pmCIA(singledata, s.gridpt)
#' res.network <- cv.l0pmCIA(singedata, n.gridpt, tL=demoData$tL, n.penalty=T)
#' @export
genGridpt <- function(s=c(50, 50, 50),
                      edit = list(matrix(c(-10, 0, 10), ncol=1),matrix(c(-10, 0, 10), ncol=1),matrix(c(-10, 0, 10), ncol=1)),
                      n.steps=c(0,1,5), 
                      n.penalty=FALSE, 
                      info.lambda=NULL)
{
  kronftn = function(x,y) cbind(rep(x, each=dim(y)[1]), kronecker(rep(1,dim(x)[1]), y))
  steps <- seq(n.steps[1], n.steps[2], length=n.steps[3]+2)
  steps <- steps[-c(1, n.steps[3]+2)]
  for (i in length(edit):2)
  {
    if (i==length(edit)) edits = edit[[i]]
    edits = kronftn(edit[[i-1]], edits)
  }
  s.choice <- matrix(rep(s, dim(edits)[1]), nrow=dim(edits)[1], byrow = TRUE) + edits
  gridpt <- cbind( rep(steps, each=dim(s.choice)[1]), kronecker(rep(1,length(steps)), s.choice))
  # gridpt : number of nonzero elements and stepsize - recommended value in the paper is less than maximum eigenvalue of "B", identity matrix in our applicataion, smCIA
  if (n.penalty==TRUE)
  {
    lambdas <- genseq(info.lambda[1], info.lambda[2], len=info.lambda[3], incr=info.lambda[4])
    gridpt <- cbind( rep(lambdas, each=dim(gridpt)[1]), kronecker(rep(1,length(lambdas)), gridpt)) # columnwise: lambda, stepsize, number of nonzeros in datasets
  }else{
    tL <- NULL
    gridpt <- cbind( rep(0, dim(gridpt)[1]), gridpt)  # columnwise: lambda=0, stepsize, number of nonzeros in datasets
  }
  return(gridpt)
}

