#' Penalized multiple Co-Inertia Analysis 
#' 
#' This function conduct penalized multiple co-inertia analysis for a given set of tuning parameter values. This function esimate one set of loading vectors for a given datasets.
#' To estimate consequent loading vectors, put deflated datasets using previously estimated sets of loading vectors. 
#' @param Tdat n-by-p concatenated matrix of transformed data, \tilde{\mathbf{X}}_1, \ldots, \tilde{\mathbf{X}}_K, where p=\sum_{i=k}^K p_i.
#' @param ps K-by-1 vector containing all dimensions of each dataset.
#' @param s K-by-1 vector containing number of desired nonzero element in each dataset.
#' @param step A scaler value used as a stepsize for the gradient descent algorithm. Default value is \code{0.1}.
#' @param n.penalty Indicator variable to let the function know whether user wants to incorporate structured information existing among data matrices. Default value is \code{FALSE}.
#' @param tL A length K list object containing transformed laplacian matrix for network penalties. k-th element of \code{tL} contains a transformed laplacian matrix of the network information for the k-th data matrix.
#' @param lambda A scaler value used as a tuning parameter for the network penalty.
#' @param threshold A scaler value used as a threshold for the algorithm. Default value is \code{1e-3}.
#' @param max_iter A scaler value used as a maximum number of iterations of the algorithm. Default value is \code{300}.
#' @return This function returns a list object.
#' \item{a}{A length-K list contains the estimated loading vectors of K data matrices.}
#' \item{b}{A n-by-1 vector, the estimated synthetic reference vector.}
#' \item{mark}{An indicator that tells about whether the algorithm was forced to finish due to the oscillation. \code{1} means the }
#' @seealso \code{\link{mCIA}}, \code{\link{cv.l0pmCIA}}, \code{\link{plot.pmCIA}}
#' @examples
#' library(pmCIA)
#' data(demoData)
#' single.data <- cbind(demoData$X1[1:200,],demoData$X2[1:200,], demoData$X3[1:200,])
#' ps <- unlist(lapply(data, function(x) dim(x)[2]) ) 
#' s <- rep(50, length(ps))
#' lambda <- 0.8
#' res.spare <- l0pmCIA(single.data, ps, s)
#' res.network <- l0pmCIA(single.data, ps, s, n.penalty=T, tL=demoData$tL, lambda=lambda) 
#' @export
l0pmCIA = function(Tdat, ps, s, step=0.1, n.penalty = FALSE, tL = NULL, lambda=NULL, threshold = 1e-3, max_iter=300)
{
  n <- dim(Tdat)[1]
  K <- length(ps)
  a <- NULL
  for (i in 1:K)
  {
    ind1 <- sum(ps[1:i]) - ps[i] + 1
    ind2 <- sum(ps[1:i])
    eig.res <- eigen(t(Tdat[,ind1:ind2])%*%Tdat[,ind1:ind2])
    m.ind <- which.max(eig.res$values)
    a[[i]] <- eig.res$vectors[,m.ind]
  }
  
  if (n.penalty == FALSE)
  {
    tL <- lapply(ps, diag)
    lambda <- 0
  }
  
  iter <- o.flag <- obj <- mark <-  0
  gap <- old.gap <- Inf
  
  while ( gap>threshold && iter<max_iter )
  {
    iter <- iter+1
    # estimate b
    mat.b <- matrix(0, nrow=n, ncol=n)
    for (i in 1:K)
    {
      ind1 <- sum(ps[1:i]) - ps[i] + 1
      ind2 <- sum(ps[1:i])
      mat.b <- mat.b + Tdat[,ind1:ind2] %*% a[[i]] %*% t(a[[i]]) %*% t(Tdat[,ind1:ind2])
    }
    b <- rARPACK::eigs(mat.b, k=1)$vectors
    
    # estimaet as
    for (i in 1:K)
    {
      ind1 <- sum(ps[1:i]) - ps[i] + 1
      ind2 <- sum(ps[1:i])
      Amat <- t(Tdat[,ind1:ind2]) %*% b %*% t(b) %*% Tdat[,ind1:ind2] - lambda*tL[[i]]
      i.iter <- flag<- 0; diff <- old.diff <- Inf
      while( diff > 1e-3 && i.iter <= 500)
      {
        i.iter <- i.iter+1
        rho  <- drop( t(a[[i]]) %*% Amat %*% a[[i]] /  (t(a[[i]]) %*% a[[i]]) )
        Cmat <- diag(ps[i]) + (step/rho) * ( Amat - rho*diag(ps[i]) )
        tmp <- Cmat %*% a[[i]] / norm(Cmat %*% a[[i]],"2")
        t.value <- sort(abs(tmp), decreasing=TRUE)[s[i]]
        tmp[which(abs(tmp)<t.value)] <- 0
        if (sum(tmp^2)!=0){
          tmp <- tmp/sqrt(sum(tmp^2))
        }
        diff <- norm(tmp-a[[i]], "2")
        # check difference and push to finish the algorith if solution is oscillating
        if (abs(abs(diff)-abs(old.diff)) < 1e-5) flag <- flag + 1
        if (flag > 30) i.iter <- 501
        old.diff <- diff
        a[[i]] <- tmp
      }
    }
    # decision and update
    obj.new <- sum(sapply(1:K, function(i) (t(b) %*% Tdat[,(sum(ps[1:i])-ps[i]+1):(sum(ps[1:i]))] %*% a[[i]])^2 ))
    gap <- abs(obj.new - obj)
    # check difference and push to finish the algorithm if solution is oscillating
    if (abs(gap-old.gap) < 1e-5) o.flag <- o.flag + 1
    if (o.flag > 50) {iter <- max_iter; mark <- mark + 1}
    old.gap <- gap
    obj <- obj.new
  }
  if (iter>=max_iter) warning("Failed to converge within the allowed time of iteration")
  return(list("a"= a, "b"=b, "mark"=ifelse(mark>0, 1, 0)))
}
