#' Generate Multiple Co-Inertia Analysis Figure
#' 
#' This function generates output plots of pmCIA. K plots corresponding to each K dataset are generated. The biggest 
#' @param res A list containing the result of pmCIA for more than equal to two sets of estimated loading vectors. First to elements are used to generate the figure.
#' @param ID A length-K list containing the variable names for each K dataset.
#' @param nlab A number of labels that is shown in the figure. First nlab variables that has absolute large values will be shown in red dot with label if supplied.
#' @return This functoin returns K figures. 
#' @seealso \code{\link{mCIA}}, \code{\link{l0pmCIA}}, \code{\link{cv.l0pmCIA}}
#' @examples 
#' library(pmCIA)
#' data(demoData)
#' s.gridpt <- genGridpt(s=c(50, 50, 50), 
#'                       edit = list(matrix(c(-10, 0, 10), ncol=1),matrix(c(-10, 0, 10), ncol=1),
#'                       matrix(c(-10, 0, 10), ncol=1)), 
#'                       n.steps=c(0,1,5))
#' n.gridpt <- genGridpt(s=c(50, 50, 50),
#'                        edit = list(matrix(c(-10, 0, 10), ncol=1),matrix(c(-10, 0, 10), ncol=1),
#'                        matrix(c(-10, 0, 10), ncol=1)),
#'                        n.steps=c(0,1,5), 
#'                        n.penalty = T, 
#'                        info.lambda = c(0,1,5,1))
#' single.data <- list(demoData$X1[1:200,],demoData$X2[1:200,], demoData$X3[1:200,])
#' ps <- unlist(lapply(data, function(x) dim(x)[2]) ) 
#' res.mCIA <- mCIA(single.data, ps)
#' IDs <- sapply(ps, function(x) 1:x) 
#' plot.pmCIA(res.mCIA, IDs)
#' @export
plot.pmCIA <- function(res, ID, nlab=20)
{
  if (length(res) == 1) 
    stop("Need two sets of loading vectors")
  nps <- length(res[[1]]$a)
  res.dist = ind.res <- list()
  for(ps in 1:nps)
  {  
    res.coef <- cbind(res[[1]]$a[[ps]], res[[2]]$a[[ps]])
    res.dist <- apply(res.coef, 1, function(x) norm(x, "2"))
    ind.res <- order(-res.dist)[1:20]
    
    xmax <- max(abs(res[[1]]$a[[ps]]))* 1.1
    ymax <- max(abs(res[[2]]$a[[ps]]))* 1.1
    lim <- max(xmax, ymax)
     
    plot(res[[1]]$a[[ps]], res[[2]]$a[[ps]], pch=20, cex=1, xlab="", ylab="", xlim=c(-xmax, xmax), ylim=c(-ymax, ymax), axes=F)
    box()
    title(xlab="PC1", ylab="PC2", line=0.2, cex.lab=0.7)
    title(main=paste("Variable Space - ", ps, "-th data"))
    abline(h=0, v=0, col="gray60")
    points(res[[1]]$a[[ps]][ind.res], res[[2]]$a[[ps]][ind.res], pch=20, cex=1, col="red")
    if(!is.null(ID)) text(res[[1]]$a[[ps]][ind.res], res[[2]]$a[[ps]][ind.res], labels=ID[[ps]][ind.res], pos=4, cex=.6, srt=0) 
  }
}